------------------------------------------------
--                  CT_Core                   --
--                                            --
-- Core addon for doing basic and popular     --
-- things in an intuitive way.                --
-- Please do not modify or otherwise          --
-- redistribute this without the consent of   --
-- the CTMod Team. Thank you.                 --
------------------------------------------------

local _G = getfenv(0);
local module = _G.CT_Core;
local NUM_CHAT_WINDOWS = NUM_CHAT_WINDOWS;

--------------------------------------------
-- Quest Levels

local displayLevels = false;

local oldGetQuestLogTitle = GetQuestLogTitle;
local questLogPrefixes = {
	[GROUP] = "+",
	[RAID] = "R",
	[RAID .. " (25)"] = "R",
	[RAID .. " (10)"] = "R",
	[PVP] = "P",
	[LFG_TYPE_DUNGEON] = "D",
};

function GetQuestLogTitle(...)
	local questLogTitleText, level, questTag, unknown, isHeader, isCollapsed, isComplete, isDaily, questID = oldGetQuestLogTitle(...);
	
	if ( displayLevels and questLogTitleText and level and level > 0 and not isHeader ) then
		local prefix = questLogPrefixes[questTag or ""] or "";
		questLogTitleText = string.format("[%d%s] %s", level, prefix, questLogTitleText);
	end
	return questLogTitleText, level, questTag, unknown, isHeader, isCollapsed, isComplete, isDaily, questID;
end

local function toggleDisplayLevels(enable)
	displayLevels = enable;
	if (QuestLogFrame:IsShown()) then
		pcall(QuestLog_Update);
	end
end

--------------------------------------------
-- Hail Mod

local function hail()
	local targetName = UnitName("target");
	if ( targetName ) then
		SendChatMessage("Hail, " .. targetName .. (((UnitIsDead("target") or UnitIsCorpse("target")) and "'s Corpse") or ""));
	else
		SendChatMessage("Hail");
	end
end

module.hail = hail;
module:setSlashCmd(hail, "/hail");

--------------------------------------------
-- Block trades when bank or guild bank is open

do
	local blockOption;
	local blockOriginal;
	local blockcvar = "blockTrades";

	local function restoreBlockState()
		-- Restore blocking to its original state, which could be disabled or enabled.
		if (blockOriginal) then
			SetCVar(blockcvar, blockOriginal);
			blockOriginal = nil;
		end
	end

	local function enableBlockState()
		-- Change blocking state to enabled.
		if (blockOriginal == nil) then
			-- Save the original blocking state before we change it.
			blockOriginal = GetCVar(blockcvar);
		end
		-- Blocking is now enabled.
		SetCVar(blockcvar, "1");
	end

	-- If leaving the world, or the window is being closed, then restore
	-- blocking to its original state.
	module:regEvent("PLAYER_LEAVING_WORLD", restoreBlockState);
	module:regEvent("BANKFRAME_CLOSED", restoreBlockState);
	module:regEvent("GUILDBANKFRAME_CLOSED", restoreBlockState);

	-- If the bank frame has just opened, and the user wants to block while
	-- at the bank, then start blocking.
	module:regEvent("BANKFRAME_OPENED", function()
		if (blockOption) then
			enableBlockState();
		end
	end);

	-- If the guild bank frame has just opened, and the user wants to block while
	-- at the guild bank, then start blocking.
	module:regEvent("GUILDBANKFRAME_OPENED", function()
		if (blockOption) then
			enableBlockState();
		end
	end);

	-- Configure blocking option.
	module.configureBlockTradesBank = function(block)
		blockOption = block; -- Save the option's value in a local variable
		if (blockOption) then
			-- User wants to block trades while at this window.
			-- If the frame is currently shown, then start blocking.
			if ( (BankFrame and BankFrame:IsShown()) or (GuildBankFrame and GuildBankFrame:IsShown()) ) then
				enableBlockState();
			end
		else
			-- User does not want to block trades while at this window.
			-- If we are currently blocking trades (ie. if we have the original
			-- blocking state saved), then restore to the original blocking state.
			if (blockOriginal) then
				restoreBlockState();
			end
		end
	end
end

--------------------------------------------
-- Common code for Tooltip Reanchoring

	local tooltipText = "Left-click to drag\nRight-click to rotate.";
	local tooltipsTooltipText = "|c00FFFFFFTooltips|r\n" .. tooltipText;
	local function getPositions(anchor, anchorMode)
		if ( anchorMode == 2 ) then
			return "ANCHOR_CURSOR";
		else
			if ( anchor == 1 ) then
				return "ANCHOR_NONE", "TOPLEFT", "BOTTOMLEFT", "Top Left";
			elseif ( anchor == 2 ) then
				return "ANCHOR_NONE", "TOPRIGHT", "BOTTOMRIGHT", "Top Right";
			elseif ( anchor == 3 ) then
				return "ANCHOR_NONE", "BOTTOMRIGHT", "TOPRIGHT", "Bottom Right";
			elseif ( anchor == 4 ) then
				return "ANCHOR_NONE", "BOTTOMLEFT", "TOPLEFT", "Bottom Left";
			end
		end
	end
	local function onMouseDownFunc(self, button)
		if ( button == "LeftButton" ) then
			module:moveMovable(self.movable);
		end
	end
	local function anchorFrameSkeleton()
		-- Updates the text

		return "button#st:HIGH#tl:mid:350:-200#s:100:30", {
			"backdrop#tooltip#0:0:0:0.75",
			"font#v:GameFontNormal#i:text",
			["onleave"] = module.hideTooltip,
			["onmousedown"] = onMouseDownFunc
		};
	end

-- End common code

--------------------------------------------
-- Tooltip Reanchoring

local tooltipAnchor = 1;
local tooltipAnchorMode = 0;
local tooltipAnchorDisplay = true;
local tooltipAnchorFrame;

local function updateTooltipAnchorVisibility()
	if ( tooltipAnchorFrame ) then
		if ( tooltipAnchorMode == 3 and tooltipAnchorDisplay ) then
			tooltipAnchorFrame:Show();
		else
			tooltipAnchorFrame:Hide();
		end
	end
end

local function updateTooltipText(self)
	local _, text;
	_, _, _, text = getPositions(tooltipAnchor, tooltipAnchorMode);
	self.text:SetText(text);
end

local function createTooltipAnchorFrame()
	local movable = "TOOLTIPANCHOR";	
	tooltipAnchorFrame = module:getFrame(anchorFrameSkeleton);
	updateTooltipText(tooltipAnchorFrame);
	updateTooltipAnchorVisibility();
	
	module:registerMovable(movable, tooltipAnchorFrame);
	tooltipAnchorFrame.movable = movable;
	tooltipAnchorFrame:SetScript("OnEnter", function(self) module:displayTooltip(self, tooltipsTooltipText, true); end);
	tooltipAnchorFrame:SetScript("OnMouseUp", function(self, button)
		if ( button == "LeftButton" ) then
			module:stopMovable(self.movable);
		elseif ( button == "RightButton" ) then
			-- Update anchor & text
			tooltipAnchor = tooltipAnchor + 1;
			if ( tooltipAnchor > 4 ) then
				tooltipAnchor = 1;
			end
			module:setOption("tooltipAnchor", tooltipAnchor, true);
			updateTooltipText(self);

			-- Display tooltip & play sound
			self:GetScript("OnEnter")(self);
			PlaySound("UChatScrollButton");
		end
	end);
end

local function reanchorTooltip(tooltip, parent)
	local owner, pt, relPt = getPositions(tooltipAnchor, tooltipAnchorMode);
	tooltip:SetOwner(parent, owner);
	if ( pt ) then
		if ( not tooltipAnchorFrame ) then
			createTooltipAnchorFrame();
		end
		tooltip:ClearAllPoints();
		tooltip:SetPoint(pt, tooltipAnchorFrame, relPt);
	end
end

function CT_Core_ResetTooltipAnchor()
	if (tooltipAnchorFrame) then
		tooltipAnchorFrame:ClearAllPoints();
		tooltipAnchorFrame:SetPoint("CENTER", "UIParent", "CENTER", 0, 0);
		module:stopMovable(tooltipAnchorFrame.movable);
	end
end


function CT_Core_GameTooltip_SetDefaultAnchor(tooltip, parent, ...)
	if ( tooltip == GameTooltip and tooltipAnchorMode and tooltipAnchorMode > 1 ) then
		reanchorTooltip(tooltip, parent);
	end
end
hooksecurefunc("GameTooltip_SetDefaultAnchor", CT_Core_GameTooltip_SetDefaultAnchor);


local function setTooltipRelocationStyle(tooltipStyle)
	tooltipAnchorMode = tooltipStyle;
	updateTooltipAnchorVisibility();
end

local function toggleTooltipAnchorVisibility(show)
	tooltipAnchorDisplay = show;
	
	if ( not tooltipAnchorFrame and show ) then
		createTooltipAnchorFrame();
	else
		updateTooltipAnchorVisibility();
	end
end

local function setTooltipAnchor(anchor)
	tooltipAnchor = anchor or tooltipAnchor;
	
	if ( tooltipAnchorDisplay ) then
		if ( not tooltipAnchorFrame ) then
			createTooltipAnchorFrame();
		end
		updateTooltipText(tooltipAnchorFrame);
	end
end

--------------------------------------------
-- Tick Mod

local tickFrame;
local tickDisplayType = 1;

local tickFormatHealth_1 = "Health: %d";
local tickFormatHealth_2 = "HP/Tick: %d";
local tickFormatHealth_3 = "HP: %d";
local tickFormatMana_1 = "Mana: %d";
local tickFormatMana_2 = "MP/Tick: %d";
local tickFormatMana_3 = "MP: %d";

local tickFrameWidth;
local tickCounter = 0.05;
local lastTickHealth, lastTickMana;

local function fadeObject(self)
	local alpha = self.alpha;
	if ( alpha and alpha > 0.25 ) then
		alpha = alpha - 0.03;
		self.alpha = alpha;
		self:SetAlpha(alpha);
		return true;
	end
end

local function fadeTicks(self, elapsed)
	tickCounter = tickCounter - elapsed;
	if ( tickCounter < 0 ) then
		local fadedHealth = fadeObject(self.health);
		local fadedMana = fadeObject(self.mana);
		if ( not fadedHealth and not fadedMana ) then
			self:SetScript("OnUpdate", nil);
		end
		tickCounter = 0.05;
	end
end

local function updateTickDisplay(key, diff)
	local obj = tickFrame[key];
	obj:SetText(format(obj.strFormat, diff));
	obj:SetAlpha(1);
	obj.alpha = 1;
	
	if ( tickFrameWidth ) then
		tickFrame:SetWidth(tickFrameWidth);
	end
	
	counter = 0.05;
	tickFrame:SetScript("OnUpdate", fadeTicks);
end

local function tickFrameSkeleton()
	return "button#tl:mid:350:-200#s:90:40", {
		"backdrop#tooltip",
		"font#i:health#t:0:-8",
		"font#i:mana#b:0:8",
		["onload"] = function(self)
			self:RegisterEvent("UNIT_HEALTH");
			self:RegisterEvent("UNIT_MANA");
			self:SetBackdropColor(0, 0, 0, 0.75);
			module:registerMovable("TICKMOD", self, true);
		end,
		["onevent"] = function(self, event, unit)
			if ( unit == "player" ) then
				if ( event == "UNIT_HEALTH" ) then
					local health = UnitHealth("player");
					local diff = health-lastTickHealth;
					if ( diff > 0 ) then
						updateTickDisplay("health", diff);
					end
					lastTickHealth = health;
				else
					local mana = UnitMana("player");
					local diff = mana-lastTickMana;
					if ( diff > 0 ) then
						updateTickDisplay("mana", diff);
					end
					lastTickMana = mana;
				end
			end
		end,
		["onenter"] = function(self)
			module:displayPredefinedTooltip(self, "DRAG");
		end,
		["onleave"] = module.hideTooltip,
		["onmousedown"] = function(self, button)
			if ( button == "LeftButton" ) then
				module:moveMovable("TICKMOD");
			end
		end,
		["onmouseup"] = function(self, button)
			if ( button == "LeftButton" ) then
				module:stopMovable("TICKMOD");
			elseif ( button == "RightButton" ) then
				module:resetMovable("TICKMOD");
				self:ClearAllPoints();
				self:SetPoint("CENTER", UIParent);
			end
		end
	};
end

local function updateTickFrameOptions()
	if ( not tickFrame ) then
		return;
	end
	
	-- Height
	local _, class = UnitClass("player");
	if ( UnitPowerType("player") == 0 or class == "DRUID" ) then
		tickFrame:SetHeight(40);
	else
		tickFrame:SetHeight(30);
	end
	
	-- Width & Format
	if ( not tickDisplayType or tickDisplayType == 1 ) then
		tickFrameWidth = 90;
		tickFrame.health.strFormat = tickFormatHealth_1;
		tickFrame.mana.strFormat = tickFormatMana_1;
	elseif ( tickDisplayType == 2 ) then
		tickFrameWidth = 100;
		tickFrame.health.strFormat = tickFormatHealth_2;
		tickFrame.mana.strFormat = tickFormatMana_2;
	elseif ( tickDisplayType == 3 ) then
		tickFrameWidth = 80;
		tickFrame.health.strFormat = tickFormatHealth_3;
		tickFrame.mana.strFormat = tickFormatMana_3;
	end
end

local function toggleTick(enable)
	if ( enable ) then
		if ( not tickFrame ) then
			tickFrame = module:getFrame(tickFrameSkeleton);
		end
		tickFrame:Show();
		updateTickFrameOptions();
		lastTickHealth, lastTickMana = UnitHealth("player"), UnitMana("player");

	elseif ( tickFrame ) then
		tickFrame:Hide();
	end
end

local function setTickDisplayType(mode)
	tickDisplayType = mode;
	updateTickFrameOptions();
end

--------------------------------------------
-- Casting Bar Timer

local displayTimers;

-- Create our frames
	local countDownText = CastingBarFrame:CreateFontString(nil, "ARTWORK", "GameFontHighlight");
	countDownText:SetPoint("TOPRIGHT", 2, 5);
	countDownText:SetPoint("BOTTOMLEFT", CastingBarFrame, "BOTTOMRIGHT", -56, 2);
	CastingBarFrame.countDownText = countDownText;
	CastingBarFrame.ctElapsed = 0;
	
	local countDownTargetText = TargetFrameSpellBar:CreateFontString(nil, "ARTWORK", "GameFontHighlight");
	countDownTargetText:SetPoint("TOPRIGHT", 0, 5);
	countDownTargetText:SetPoint("BOTTOMLEFT", TargetFrameSpellBar, "BOTTOMRIGHT", -60, 0);
	TargetFrameSpellBar.countDownText = countDownTargetText;
	TargetFrameSpellBar.ctElapsed = 0;

-- Preparation for repositioning the normal texts
	local castingBarText = CastingBarFrameText;
	local castingBarTargetText = TargetFrameSpellBarText;

-- Hook the CastingBarFrame's OnUpdate
function CT_Core_CastingBarFrame_OnUpdate(self, secondsElapsed)
	if (not self.ctElapsed) then
		return;
	end

	local elapsed = ( self.ctElapsed or 0 ) - secondsElapsed;
	if ( elapsed < 0 ) then
		if ( displayTimers ) then
			-- We need to update
			if ( self.casting ) then
				self.countDownText:SetText(format("(%.1fs)", max(self.maxValue - self.value,0)));
			elseif ( self.channeling ) then
				self.countDownText:SetText(format("(%.1fs)", max(self.value,0)));
			else
				self.countDownText:SetText("");
			end
		end
		self.ctElapsed = 0.1;
	else
		self.ctElapsed = elapsed;
	end
end
hooksecurefunc("CastingBarFrame_OnUpdate", CT_Core_CastingBarFrame_OnUpdate);

local function toggleCastingTimers(enable)
	displayTimers = enable;
	castingBarText:ClearAllPoints();
	castingBarTargetText:ClearAllPoints();
	if ( enable ) then
		countDownText:Show();
		countDownTargetText:Show();
		
		castingBarText:SetPoint("TOPLEFT", 0, 5);
		castingBarText:SetPoint("BOTTOMRIGHT", countDownText, "BOTTOMLEFT", 10, 0);
		
		castingBarTargetText:SetPoint("TOPLEFT", 2, 5);
		castingBarTargetText:SetPoint("BOTTOMRIGHT", countDownTargetText, "BOTTOMLEFT", 10, 0);
	else
		countDownText:Hide();
		countDownTargetText:Hide();
		
		castingBarText:SetWidth(0);
		castingBarText:SetPoint("TOP", 0, 5);
		
		castingBarTargetText:SetWidth(0);
		castingBarTargetText:SetPoint("TOP", 0, 5);
	end
end

--------------------------------------------
-- Player Notes

local guildNotes, friendNotes, ignoreNotes;
local playerNoteButtons;

local function updatePlayerNotes(save)
	if ( save ) then
		module:setOption("guildNotes", guildNotes, true);
		module:setOption("friendNotes", friendNotes, true);
		module:setOption("ignoreNotes", ignoreNotes, true);
	else
		guildNotes, friendNotes, ignoreNotes =
			module:getOption("guildNotes"),
			module:getOption("friendNotes"),
			module:getOption("ignoreNotes");
	end
end

local function updateNote(self, playerName)
	local text = (self.wideEditBox):GetText();
	if ( text ~= "" ) then
		currentNoteType[playerName] = text;
	else
		currentNoteType[playerName] = nil;
	end
	if ( currentNoteType == guildNotes ) then
		GuildStatus_Update();
	elseif ( currentNoteType == friendNotes ) then
		FriendsList_Update();
	else
		IgnoreList_Update();
	end
	updatePlayerNotes(true);
end

local function getNoteDialogTable()
	return {
		text = module:getText("EDITING"),
		button1 = TEXT(ACCEPT),
		button2 = TEXT(CANCEL),
		hasEditBox = 1,
		hasWideEditBox = 1,
		maxLetters = 128,
		whileDead = 1,
		OnAccept = function(self, playerName)
			updateNote(self, playerName);
		end,
		timeout = 0,
		EditBoxOnEnterPressed = function(self, playerName)
			updateNote(self:GetParent(), playerName);
			self:GetParent():Hide();
		end,
		EditBoxOnEscapePressed = function(self, playerName)
			self:GetParent():Hide();
		end,
		hideOnEscape = 1
	};
end

local function editNote(playerName, noteType)
	if ( not StaticPopupDialogs["PLAYERNOTE_EDIT"] ) then
		StaticPopupDialogs["PLAYERNOTE_EDIT"] = getNoteDialogTable();
	end
	
	local coloredName;
	if ( noteType == ignoreNotes ) then
		coloredName = "|c00FF0000"..playerName.."|r";
	else
		coloredName = "|c0000FF00"..playerName.."|r";
	end
	
	local dialog = StaticPopup_Show("PLAYERNOTE_EDIT", coloredName);
	local staticPopupName = StaticPopup_Visible("PLAYERNOTE_EDIT");
	local editBox = _G[staticPopupName.."WideEditBox"];
	
	currentNoteType = noteType;
	dialog.data = playerName;
	editBox:SetText(noteType[playerName] or "");
	editBox:HighlightText();
end

local function playerNoteSkeleton()
	return "button#s:16:16#st:TOOLTIP#cache", {
		["onload"] = function(self)
			local normalTexture = self:CreateTexture();
			local highlightTexture = self:CreateTexture();
			normalTexture:SetAllPoints(self);
			highlightTexture:SetAllPoints(self);
			highlightTexture:SetBlendMode("ADD");
			
			self.normalTexture = normalTexture;
			self:SetNormalTexture(normalTexture);
			self:SetHighlightTexture(highlightTexture);
			self:SetDisabledTexture("Interface\\Buttons\\UI-GuildButton-PublicNote-Disabled");
			normalTexture:SetTexture("Interface\\Buttons\\UI-GuildButton-PublicNote-Up");
			highlightTexture:SetTexture("Interface\\Buttons\\UI-GuildButton-PublicNote-Up");
		end,
		["onclick"] = function(self)
			editNote(self.name, self.type);
		end,
		["onenter"] = function(self)
			local tooltip = GameTooltip;
			tooltip:SetOwner(self, "ANCHOR_RIGHT");
			tooltip:ClearLines();
			tooltip:AddLine(module:getText("CLICKEDIT"), 1, 0.7, 0);
			tooltip:AddLine(self.note);
			tooltip:Show();
		end,
		["onleave"] = module.hideTooltip
	};
end

local function getPlayerNoteButton(index)
	local obj = playerNoteButtons[index];
	if ( not obj ) then
		obj = module:getFrame(playerNoteSkeleton);
		playerNoteButtons[index] = obj;
	end
	return obj;
end

local function updateDisplay(offset, maxNum, framePrefix, frameOffset, tbl, nameFunc, parent)
	local numButtons, index, frame, name = 0;
	
	if ( playerNoteButtons ) then
		numButtons = #playerNoteButtons;
	else
		playerNoteButtons = { };
	end
	
	for i=1, max(maxNum, numButtons), 1 do
		btn = getPlayerNoteButton(i);
		if ( i > maxNum ) then
			btn:Hide();
		else
			index = offset + i;
			name = nameFunc(index);
			frame = _G[framePrefix..i];
			
			local note = tbl[name];
			if ( note ) then
				btn.note = note;
				btn.normalTexture:SetVertexColor(1.0, 1.0, 1.0);
			else
				btn.note = "";
				btn.normalTexture:SetVertexColor(0.5, 0.5, 0.5);
			end

			btn.type = tbl;
			btn.name = name;
			
			btn:SetParent(parent or frame);
			btn:ClearAllPoints();
			btn:SetPoint("RIGHT", frame, "LEFT", frameOffset, 0);
			btn:SetFrameLevel(frame:GetFrameLevel()+1);
			btn:Show();
		end
	end
end

-- Hooks for knowing which icons to display
function CTCore_FriendsList_Update(...)
	if ( not FriendsListFrame:IsVisible() ) then
		return;
	end
	
	local maxDisplay = FRIENDS_TO_DISPLAY;
	local maxNum = GetNumFriends();
	if ( maxNum > 0 and not friendNotes ) then
		friendNotes = { };
	end
	
	updateDisplay(FauxScrollFrame_GetOffset(FriendsFrameFriendsScrollFrame),
		min(maxNum, maxDisplay), "FriendsFrameFriendButton", 
		( ( maxDisplay >= maxNum ) and 315 ) or 290, friendNotes, GetFriendInfo);
end
hooksecurefunc("FriendsList_Update", CTCore_FriendsList_Update);

function CTCore_IgnoreList_Update(...)
	if ( not IgnoreListFrame:IsVisible() ) then
		return;
	end
	
	local maxDisplay = IGNORES_TO_DISPLAY;
	local maxNum = GetNumIgnores();
	if ( maxNum > 0 and not ignoreNotes ) then
		ignoreNotes = { };
	end
	
	updateDisplay(FauxScrollFrame_GetOffset(FriendsFrameFriendsScrollFrame),
		min(maxNum, maxDisplay), "FriendsFrameIgnoreButton", 
		( ( maxDisplay >= maxNum ) and 315 ) or 290, ignoreNotes, GetIgnoreName);
end
hooksecurefunc("IgnoreList_Update", CTCore_IgnoreList_Update);

function CTCore_GuildStatus_Update(...)
	if ( not GuildFrame:IsVisible() ) then
		return;
	end
	
	local framePrefix;
	if ( FriendsFrame.playerStatusFrame ) then
		framePrefix = "GuildFrameGuildStatusButton";
	else
		framePrefix = "GuildFrameButton";
	end
	
	local maxDisplay = GUILDMEMBERS_TO_DISPLAY;
	local maxNum = GetNumGuildMembers();
	if ( maxNum > 0 and not guildNotes ) then
		guildNotes = { };
	end
	
	updateDisplay(FauxScrollFrame_GetOffset(GuildListScrollFrame), min(maxNum, maxDisplay),
		framePrefix, ( ( maxDisplay >= maxNum ) and 320 ) or 303,
		guildNotes, GetGuildRosterInfo, GuildFrame);
end
hooksecurefunc("GuildStatus_Update", CTCore_GuildStatus_Update);

--------------------------------------------
-- Alt+Right-Click to buy full stack

local function CT_Core_MerchantItemButton_OnModifiedClick(self, ...)
	local merchantAltClickItem = module:getOption("merchantAltClickItem") ~= false;  -- if option is nil then default to true
	if (merchantAltClickItem and IsAltKeyDown()) then
		local id = self:GetID();
		local maxStack = GetMerchantItemMaxStack(id);
		local money = GetMoney();
		local _, _, price, quantity = GetMerchantItemInfo(id);
		
		if ( maxStack == 1 and quantity > 1 ) then
			-- We need to check max stack count
			local _, _, _, _, _, _, _, stackCount = GetItemInfo(GetMerchantItemLink(id));
			if ( stackCount and stackCount > 1 ) then
				maxStack = floor(stackCount/quantity);
			end
		end
	
		if ( maxStack*price > money ) then
			maxStack = floor(money/price);
		end

		BuyMerchantItem(id, maxStack);
	end
end

hooksecurefunc("MerchantItemButton_OnModifiedClick", CT_Core_MerchantItemButton_OnModifiedClick);

--------------------------------------------
-- Alt Left Click to add item to auctions frame.

local function CT_Core_AddToAuctions(self, button)
	if (button == "LeftButton" and IsAltKeyDown()) then
		if (AuctionFrame and AuctionFrame:IsShown()) then
			local auctionAltClickItem = module:getOption("auctionAltClickItem");
			if (auctionAltClickItem and
				not CursorHasItem()
			) then
				if (not AuctionFrameAuctions:IsVisible()) then
					-- Switch to the "auctions" tab.
					AuctionFrameTab_OnClick(AuctionFrameTab3, 3);
				end
				-- Pickup and place item in the auction sell button.
				local bag, item = self:GetParent():GetID(), self:GetID();
				PickupContainerItem(bag, item);
				ClickAuctionSellItemButton(AuctionsItemButton, "LeftButton");
				AuctionsFrameAuctions_ValidateAuction();
				return true;
			end
		end
	end
	return false;
end

--------------------------------------------
-- Alt Left Click to initiate trade or add item to trade window.

local CT_Core_AddToTrade;

do
	local prepareTrade;
	local addItemToTrade;

	do
		local prepBag, prepItem, prepPlayer;
		local function clearTrade()
			prepBag, prepItem, prepPlayer = nil;
		end
		
		prepareTrade = function(bag, item, player) -- Local
			prepBag, prepItem, prepPlayer = bag, item, player;
			module:schedule(3, clearTrade);
		end

		addItemToTrade = function(bag, item)
			local slot = TradeFrame_GetAvailableSlot();
			if (slot) then
				PickupContainerItem(bag, item);
				ClickTradeButton(slot);
			end
		end

		module:regEvent("TRADE_SHOW", function()
			if ( prepBag and prepItem and UnitName("target") == prepPlayer ) then
				addItemToTrade(prepBag, prepItem);
			end
			clearTrade();
		end);
	end

	CT_Core_AddToTrade = function(self, button)
		if (button == "LeftButton" and IsAltKeyDown()) then
			if (TradeFrame) then
				if (not TradeFrame:IsShown()) then
					local tradeAltClickOpen = module:getOption("tradeAltClickOpen");
					if (tradeAltClickOpen and
						not CursorHasItem() and 
						UnitExists("target") and
						CheckInteractDistance("target", 2) and
						UnitIsFriend("player", "target") and
						UnitIsPlayer("target")
					) then
						-- Initiate a trade and in a few seconds pickup and add the item to the trade window.
						local bag, item = self:GetParent():GetID(), self:GetID();
						InitiateTrade("target");
						prepareTrade(bag, item, UnitName("target"));
						return true;
					end
				else
					local tradeAltClickAdd = module:getOption("tradeAltClickAdd");
					if (tradeAltClickAdd and
						not CursorHasItem()
					) then
						-- Pickup and add an item to the trade window.
						local bag, item = self:GetParent():GetID(), self:GetID();
						addItemToTrade(bag, item);
						return true;
					end
				end
			end
		end
		return false;
	end
end

--------------------------------------------
-- Handle clicks on item in a container frame.
-- Currently used by CT_Core and CT_MailMod.

local cfibomcTable = {};

function CT_Core_ContainerFrameItemButton_OnModifiedClick_Register(func)
	cfibomcTable[func] = true;
end

function CT_Core_ContainerFrameItemButton_OnModifiedClick_Unregister(func)
	cfibomcTable[func] = nil;
end

local function CT_Core_ContainerFrameItemButton_OnModifiedClick(self, button)
	-- Test registered functions
	for func, value in pairs(cfibomcTable) do
		if (func(self, button)) then
			return;
		end
	end
	-- Test for the Add To Trade function last, since this one
	-- doesn't require a particular frame to be open (unless you're
	-- adding to an open trade frame).
	CT_Core_AddToTrade(self, button);
end

hooksecurefunc("ContainerFrameItemButton_OnModifiedClick", CT_Core_ContainerFrameItemButton_OnModifiedClick);

CT_Core_ContainerFrameItemButton_OnModifiedClick_Register(CT_Core_AddToAuctions);


--------------------------------------------
-- Hide Gryphons

local gryphonLoop;

local function toggleGryphons(hide)
	if (gryphonLoop) then
		-- Ensure no infinite loop.
		gryphonLoop = nil;
		return;
	end
	-- Hide/Show the gryphons
	if ( hide ) then
		MainMenuBarLeftEndCap:Hide();
		MainMenuBarRightEndCap:Hide();
	else
		MainMenuBarLeftEndCap:Show();
		MainMenuBarRightEndCap:Show();
	end
	if (CT_BottomBar) then
		-- CT_BottomBar is loaded, and it may also have an option
		-- to hide the gryphons.
		local optCore = "hideGryphons";
		local optBott = "hideGryphons";
		if (type(module.frame) == "table") then
			-- Update our "hide gryphons" option checkbox.
			-- This is needed for those times when CT_BottomBar calls us
			-- to change the CT_Core "hide gryphons" option.
			local cb = module.frame.section1[optCore];
			cb:SetChecked(hide);
		end
		if (CT_BottomBar:getOption(optBott) ~= module:getOption(optCore)) then
			-- Change CT_BottomBar's "hide gryphons" option.
			gryphonLoop = true;
			CT_BottomBar:setOption(optBott, hide, true);
			gryphonLoop = nil;
		end
	end
end

--------------------------------------------
-- Hide World Map Minimap Button

local function toggleWorldMap(hide)
	if ( hide ) then
		MiniMapWorldMapButton:Hide();
	else
		MiniMapWorldMapButton:Show();
	end
end

--------------------------------------------
-- Movable casting bar

local castingbarAnchorFrame;
local castingbarEnabled;
local castingbarMovable;
local castingbarAlwaysFrame;

local function castingbar_Reanchor()
	CastingBarFrame:ClearAllPoints();
	CastingBarFrame:SetPoint("CENTER", castingbarAnchorFrame, "CENTER", 0, -2);
end

local function CT_Core_Other_castingbar_UIParent_ManageFramePositions()
	if (castingbarEnabled) then
		castingbar_Reanchor();
	end
end

local function castingbar_onMouseDownFunc(self, button)
	if ( button == "LeftButton" ) then
		module:moveMovable(self.movable);
	end
end

local function castingbar_FrameSkeleton()
	return "button#st:HIGH#tl:mid:350:-200#s:100:30", {
		"tooltip#0:0:0:0.75",
		"font#v:GameFontNormal#i:text",
		["onleave"] = module.hideTooltip,
		["onmousedown"] = castingbar_onMouseDownFunc,
	};
end

local function castingbar_AlwaysVisibility(self)
	if (not self) then
		self = CastingBarFrame;
	end
	if (self.channeling or self.casting) then
		return;
	end
	if (not castingbarAlwaysFrame) then
		self:Hide();
		return;
	end
	if (not (castingbarEnabled and castingbarMovable)) then
		self:Hide();
		castingbarAlwaysFrame:Hide();
		return;
	end
	self:SetAlpha(1);
	self:SetStatusBarColor(1.0, 0.7, 0.0);
	local selfName = self:GetName();
	local barText = _G[selfName.."Text"];
	if ( barText ) then
		barText:SetText("");
	end
	self:Show();
	castingbarAlwaysFrame:Show();
end

local function castingbar_CreateAnchorFrame()
	local movable = "CASTINGBARANCHOR2";	
	castingbarAnchorFrame = module:getFrame(castingbar_FrameSkeleton, UIParent, "CT_Core_CastingBarAnchorFrame");
	
	module:registerMovable(movable, castingbarAnchorFrame, true);
	castingbarAnchorFrame.movable = movable;
	castingbarAnchorFrame:SetScript("OnEnter", function(self)
		module:displayTooltip(self, "|c00FFFFFFCasting Bar|r\nLeft-click to drag.\nRight-click to reset.");
	end);
	castingbarAnchorFrame:SetScript("OnMouseUp", function(self, button)
		if ( button == "LeftButton" ) then
			module:stopMovable(self.movable);
		elseif ( button == "RightButton" ) then
			CT_Core_CastingBarAnchorFrame:ClearAllPoints();
			CT_Core_CastingBarAnchorFrame:SetPoint("BOTTOM", UIParent, "BOTTOM", 0, 124);
		end
	end);
	castingbarAnchorFrame:SetScript("OnEvent", function(self, event, arg1, ...)
		if (event == "PLAYER_ENTERING_WORLD") then
			hooksecurefunc("UIParent_ManageFramePositions", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			-- These are required for Blizzard xml scripts that use this syntax to
			-- call UIParent_ManageFramePositions. This xml syntax does not call our
			-- secure hook of UIParent_ManageFramePositions, so we have to explicitly
			-- hook anything that calls it to ensure our function gets called.
			-- 	<OnShow function="UIParent_ManageFramePositions"/>
			-- 	<OnHide function="UIParent_ManageFramePositions"/>
			ShapeshiftBarFrame:HookScript("OnShow", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			ShapeshiftBarFrame:HookScript("OnHide", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			PossessBarFrame:HookScript("OnShow", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			PossessBarFrame:HookScript("OnHide", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			DurabilityFrame:HookScript("OnShow", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			DurabilityFrame:HookScript("OnHide", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			MainMenuBarMaxLevelBar:HookScript("OnShow", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			MainMenuBarMaxLevelBar:HookScript("OnHide", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			MultiCastActionBarFrame:HookScript("OnShow", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			MultiCastActionBarFrame:HookScript("OnHide", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			PetActionBarFrame:HookScript("OnShow", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			PetActionBarFrame:HookScript("OnHide", CT_Core_Other_castingbar_UIParent_ManageFramePositions);
			ReputationWatchBar:HookScript("OnHide", CT_Core_Other_castingbar_UIParent_ManageFramePositions);

			-- By now GetCVar("uiScale") has a value, so if Blizzard_CombatLog is already loaded
			-- then it won't cause an error when it tries to multiply by the uiScale.
			UIParent_ManageFramePositions();
		end
	end);
	castingbarAnchorFrame:SetWidth(CastingBarFrame:GetWidth() + 6);
	castingbarAnchorFrame:SetHeight(28);
	castingbarAnchorFrame:Hide();
	castingbarAnchorFrame:SetParent(CastingBarFrame);
	castingbarAnchorFrame:RegisterEvent("PLAYER_ENTERING_WORLD");

	castingbarAlwaysFrame = CreateFrame("Frame");
	castingbarAlwaysFrame:SetScript("OnUpdate", function(self, elapsed)
		castingbar_AlwaysVisibility();
	end);
end

local function castingbar_UpdateAnchorVisibility()
	if (not castingbarAnchorFrame) then
		castingbar_CreateAnchorFrame();
	end
	if (castingbarEnabled and castingbarMovable) then
		castingbarAnchorFrame:Show();
	else
		castingbarAnchorFrame:Hide();
	end
end

local function castingbar_ToggleMovable(movable)
	castingbarMovable = movable;
	castingbar_UpdateAnchorVisibility();
	castingbar_AlwaysVisibility();
end

local function castingbar_ToggleStatus(enable)
	castingbarEnabled = enable;
	castingbar_UpdateAnchorVisibility();
	castingbar_AlwaysVisibility();
	if (castingbarEnabled) then
		castingbar_Reanchor();
	else
		-- When entering the world for the FIRST time after starting the game, GetCVar("uiScale")
		-- returns nil when CT_Core loads (ie. at ADDON_LOADED event time). The game hasn't had
		-- time to update the setting yet. This is also the case for GetCVarBool("scriptErrors").
		--
		-- When the Blizzard_CombatLog addon gets loaded, it hooks the FCF_DockUpdate function
		-- which gets called by the UIParent_ManageFramePositions function in UIParent.lua.
		--
		-- If there is an addon that loads before CT_Core and causes the Blizzard_CombatLog addon
		-- to load, then we want to avoid calling UIParent_ManageFramePositions while GetCVar("uiScale")
		-- is nil. If we do call it when the uiScale is nil, then the Blizzard_CombatLog code will cause an error
		-- when it gets to the Blizzard_CombatLog_AdjustCombatLogHeight() function in Blizzard_CombatLog.lua.
		-- That code tries to multiply by GetCVar("uiScale"), and since it is still nil then there will
		-- be an error.
		--
		-- Blizzard's code won't display the error (see BasicControls.xml) because GetCVarBool("scriptErrors")
		-- is still nil when CT_Core loads. The user won't see the error unless they have an addon that loads
		-- before CT_Core and traps and displays errors.
		--
		-- To avoid this error we will only call UIParent_ManageFramePositions() when the uiScale has
		-- a value. This is the place in this addon where UIParent_ManageFramePositions() may get called
		-- at ADDON_LOADED time by CT_Libary (during the "init" options step).

		CastingBarFrame:ClearAllPoints();
		if (GetCVar("uiScale")) then
			UIParent_ManageFramePositions();
		end
	end
end

--------------------------------------------
-- Open/close bags when bank frame opens/closes.

do
	-- Blizzard's code:
	--   When the bank frame opens, they do not open the backpack.
	--   When closing the bank frame via the close button, an open backpack will remain open.
	--   When closing the bank frame via the ESC key, an open backpack will close.

	local function bankOpened()
		local bankOpenBags = module:getOption("bankOpenBags");
		if (bankOpenBags) then
			-- Open backpack (0), bags (1 to 4), and bank bags (5 to 11)
			for i = 0, 11 do
				OpenBag(i);
			end
		end
	end

	local function bankClosed()
		local bankCloseBags = module:getOption("bankCloseBags");
		if (bankCloseBags) then
			-- Close backpack (0), bags (1 to 4), and bank bags (5 to 11)
			for i = 0, 11 do
				CloseBag(i);
			end
		end
	end

	module:regEvent("BANKFRAME_OPENED", bankOpened);
	module:regEvent("BANKFRAME_CLOSED", bankClosed);
end

--------------------------------------------
-- Open/close bags when guild bank frame opens/closes.

do
	-- Blizzard's code:
	--   When the guild bank frame opens, they do not open any containers.
	--   When closing the guild bank frame via the close button, any open containers will remain open.
	--   When closing the bank frame via the ESC key, any open containers will close.

	local function gbankOpened()
		local gbankOpenBags = module:getOption("gbankOpenBags");
		if (gbankOpenBags) then
			-- Open backpack (0) and bags (1 to 4)
			for i = 0, 4 do
				OpenBag(i);
			end
		end
	end

	local function gbankClosed()
		local gbankCloseBags = module:getOption("gbankCloseBags");
		if (gbankCloseBags) then
			-- Close backpack (0) and bags (1 to 4)
			for i = 0, 4 do
				CloseBag(i);
			end
		end
	end

	module:regEvent("GUILDBANKFRAME_OPENED", gbankOpened);
	module:regEvent("GUILDBANKFRAME_CLOSED", gbankClosed);
end

--------------------------------------------
-- Open/close bags when merchant frame opens/closes.

do
	-- Blizzard's code:
	--   When the merchant frame opens, they open the backpack.
	--   When closing the merchant frame via the close button, the backpack
	--       will close if it was not open when the merchant frame was opened.
	--   When closing the merchant frame via the ESC key, an open backpack will close.

	local function merchantOpened(event)
		local merchantOpenBags = module:getOption("merchantOpenBags");
		if (merchantOpenBags) then
			-- Game handles opening of backpack if not already open.
			-- Open bags (1 to 4)
			for i = 1, 4 do
				OpenBag(i);
			end
		end
	end

	local function merchantClosed()
		local merchantCloseBags = module:getOption("merchantCloseBags");
		if (merchantCloseBags) then
			-- Close backpack (0) and bags (1 to 4)
			for i = 0, 4 do
				CloseBag(i);
			end
		end
	end

	module:regEvent("MERCHANT_SHOW", merchantOpened);
	module:regEvent("MERCHANT_CLOSED", merchantClosed);
end

--------------------------------------------
-- Block duel requests

local duelsBlocked;

local function duelRequested(event, player)
	if (duelsBlocked) then
		if (module:getOption("blockDuelsMessage")) then
			print(format("Blocked duel request from %s.", tostring(player or UNKNOWN)));
		end
		CancelDuel();
		StaticPopup_Hide("DUEL_REQUESTED");
	end
end

module:regEvent("DUEL_REQUESTED", duelRequested);

local function configureDuelBlockOption(value)
	if (value) then
		duelsBlocked = true;
		UIParent:UnregisterEvent("DUEL_REQUESTED");
	else
		if (duelsBlocked) then
			duelsBlocked = false;
			UIParent:RegisterEvent("DUEL_REQUESTED");
		end
	end
end

--------------------------------------------
-- General Initializer

local modFunctions = {
	["castingTimers"] = toggleCastingTimers,
	["questLevels"] = toggleDisplayLevels,
	["blockBankTrades"] = module.configureBlockTradesBank,
	["tickMod"] = toggleTick,
	["tickModFormat"] = setTickDisplayType,
	["tooltipRelocation"] = setTooltipRelocationStyle,
	["tooltipRelocationAnchor"] = toggleTooltipAnchorVisibility,
	["tooltipAnchor"] = setTooltipAnchor,
	["hideGryphons"] = toggleGryphons,
	["hideWorldMap"] = toggleWorldMap,
	["castingbarEnabled"] = castingbar_ToggleStatus,
	["castingbarMovable"] = castingbar_ToggleMovable,
	["blockDuels"] = configureDuelBlockOption,
};

local modFunctionsTrue = {
	-- ["name"] = function,
};

module.modupdate = function(self, type, value)
	if ( type == "init" ) then
		for key, value in pairs(modFunctions) do
			value(self:getOption(key), key);
		end
		
		for key, value in pairs(modFunctionsTrue) do
			value(self:getOption(key) ~= false, key);
		end
		updatePlayerNotes();
	else
		local func = modFunctions[type];
		if ( func ) then
			func(value, type);
		else
			func = modFunctionsTrue[type];
			if ( func ) then
				func(value ~= false, type);
			end
		end
	end
end
